//
//  SvmPredictor.cpp
//  SvmPredictor
//
//  Created by 熊田 聖也 on 12/06/03.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#include "SvmPredictor.h"
#include <iostream>
#include <shogun/features/Labels.h>
#include <shogun/features/SimpleFeatures.h>
#include <shogun/kernel/GaussianKernel.h>
#include <shogun/kernel/Chi2Kernel.h>
#include <shogun/kernel/HistogramIntersectionKernel.h>
#include <shogun/classifier/svm/LibSVM.h>
#include <shogun/base/init.h>
#include <shogun/io/SGIO.h>
#include <shogun/io/AsciiFile.h>
#include <shogun/io/SerializableAsciiFile.h>
#include <boost/format.hpp>
#include <opencv2/core/core.hpp>
#include "common.h"

SvmPredictor::SvmPredictor()
        : svm_(0) {}
        
SvmPredictor::~SvmPredictor()
{
        if ( svm_ != 0 ) {
                SG_UNREF(svm_);
        }
}

bool SvmPredictor::load(const std::string& svm_filename)
{       
        if ( svm_ != 0 ) {
                SG_UNREF(svm_);
        }
                
        svm_ = new shogun::CLibSVM();
        shogun::CSerializableAsciiFile svm_file(const_cast<char*>(svm_filename.c_str()), 'r');
        if ( !svm_->load_serializable(&svm_file) ) {
                std::cout << "unable to load svm file\n";
                return false;
        }
        return true;
}

// test ok
bool SvmPredictor::apply(
        const std::string&      data_path, 
        const std::string&      labels_path
) const
{
        shogun::CSimpleFeatures<float64_t> features;
        shogun::CSerializableAsciiFile features_file(const_cast<char*>(data_path.c_str()), 'r');
        if ( !features.load_serializable(&features_file) ) {
                std::cout << "unable to load features file\n";
                return false;
        }
        
        apply(&features, labels_path);
        return true;
}

void SvmPredictor::apply(
        shogun::CSimpleFeatures<float64_t>*     features, 
        const std::string&                      labels_path
) const
{
        shogun::CLabels* labels = svm_->apply(features);
        shogun::CAsciiFile labels_file(const_cast<char*>(labels_path.c_str()), 'w');
        labels->save(&labels_file);
}

void SvmPredictor::apply(
        const cv::Mat&          data, 
        const std::string&      labels_path
) const
{
        shogun::CSimpleFeatures<float64_t>* features = new shogun::CSimpleFeatures<float64_t>();
        common::convert(data, features);
        apply(features, labels_path);        
}

// test ok
float64_t SvmPredictor::apply(int i) const
{
        return svm_->apply(i);
}

int SvmPredictor::get_features_size() const
{
        return svm_->get_kernel()->get_num_vec_lhs();
}
        

#ifdef UNIT_TEST
#include <boost/test/unit_test.hpp>
#include <opencv2/core/core.hpp>
#include <boost/random.hpp>
#include <pstade/oven/stream_lines.hpp>
#include <boost/range/algorithm/for_each.hpp>
#include <fstream>
#include <boost/algorithm/string/classification.hpp> // is_any_of
#include <boost/algorithm/string/split.hpp>

namespace
{
        template<typename T>
        void set_test_positive_values(cv::Mat& m)
        {
                assert(cv::DataType<T>::type == m.type());
                boost::mt19937 gen;
                boost::uniform_real<> dst(20, 50);
                boost::variate_generator<boost::mt19937, boost::uniform_real<> > rnd(gen, dst);
                cv::MatIterator_<T> first = m.begin<T>();
                cv::MatIterator_<T> last = m.end<T>();
                while ( first != last ) {
                        *first = rnd();
                        ++first;
                }
        }
        
        void test_apply_0() 
        {
                const std::string svm_filename = "/Users/kumada/Projects/scenerecognizer/SvmTrainer/SvmTrainer/src/svm.bin";
                const std::string features_filename = "/Users/kumada/Projects/scenerecognizer/SvmTrainer/SvmTrainer/src/features.bin";
                SvmPredictor predictor;
                BOOST_CHECK(predictor.load(svm_filename));
                
                int rows = predictor.get_features_size();
                  
                // classify on training examples
                for ( int32_t i = 0; i < rows; ++i ) {
                        double index = predictor.apply(i);
                        if ( i < rows / 2 ) {
                                BOOST_CHECK(index > 0.0);
                        } else {
                                BOOST_CHECK(index < 0.0);
                        }
                }
        }

        
        void test_apply_1() 
        {
                const std::string svm_filename = "/Users/kumada/Projects/scenerecognizer/SvmTrainer/SvmTrainer/src/svm.bin";
                const std::string features_filename = "/Users/kumada/Projects/scenerecognizer/SvmTrainer/SvmTrainer/src/features.bin";
                SvmPredictor predictor;
                BOOST_CHECK(predictor.load(svm_filename));

                int rows = 200;
                int cols = 2;
                cv::Mat test(rows/2, cols, cv::DataType<float32_t>::type);
                set_test_positive_values<float32_t>(test);
                
                const std::string labels_filename = "/Users/kumada/Projects/scenerecognizer/SvmPredictor/SvmPredictor/src/labels.bin";
                predictor.apply(test, labels_filename);
                
                BOOST_CHECK(rows == predictor.get_features_size());
                
                // classify on training examples
                for ( int32_t i = 0; i < rows / 2; ++i ) {
                        double index = predictor.apply(i);
                        BOOST_CHECK(index > 0.0);
                }
        }

        
        
        struct Checker
        {
                Checker(int total, int part)
                        : total_(total), part_(part) {}
                
                int     total_;
                int     part_;
                
                void operator()(const std::string& line) const
                {
                        std::vector<std::string> result;
                        boost::algorithm::split(result, line, boost::is_any_of(" "));
                        for ( int i = 0; i < total_; ++i ) {
                                double k = boost::lexical_cast<double>(result[i]);
                                if ( i < part_ ) {
                                        BOOST_CHECK(k > 0.0);
                                } else {
                                        BOOST_CHECK(k < 0.0);
                                }
                        }
                }
        };
        
        void test_apply_2()
        {
                namespace oven = pstade::oven;
                
                const std::string svm_filename = "/Users/kumada/Projects/scenerecognizer/SvmTrainer/SvmTrainer/src/svm.bin";
                const std::string features_filename = "/Users/kumada/Projects/scenerecognizer/SvmTrainer/SvmTrainer/src/features.bin";
                SvmPredictor predictor;
                BOOST_CHECK(predictor.load(svm_filename));
                
                const std::string labels_filename = "/Users/kumada/Projects/scenerecognizer/SvmPredictor/SvmPredictor/src/labels.bin";
                predictor.apply(features_filename, labels_filename);
                
                int rows = predictor.get_features_size();
                std::ifstream ifs(labels_filename.c_str());
                boost::for_each(oven::stream_lines(ifs), Checker(rows, rows / 2));
        }

        void test_apply_3()
        {
                namespace oven = pstade::oven;
                
                const std::string svm_filename = "/Users/kumada/Projects/scenerecognizer/SvmTrainer/SvmTrainer/src/real_svm.bin";
                const std::string features_filename = "/Users/kumada/Projects/scenerecognizer/SvmTrainer/SvmTrainer/src/real_features.bin";
                SvmPredictor predictor;
                BOOST_CHECK(predictor.load(svm_filename));
                const std::string labels_filename = "/Users/kumada/Projects/scenerecognizer/SvmPredictor/SvmPredictor/src/real_labels.bin";
                predictor.apply(features_filename, labels_filename);
                int rows = predictor.get_features_size();
                int prows = 241;
                std::ifstream ifs(labels_filename.c_str());
                boost::for_each(oven::stream_lines(ifs), Checker(rows, prows));
        }
        
}

BOOST_AUTO_TEST_CASE(TEST_SvmPredictor)
{
        std::cout << "TEST_SvmPredictor\n";

        shogun::init_shogun_with_defaults();
        test_apply_0();
        test_apply_1();
        test_apply_2();
        test_apply_3();
        shogun::exit_shogun();
}

#endif // UNIT_TEST


