//
//  main.cpp
//  SvmPredictor
//
//  Created by 熊田 聖也 on 12/06/03.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#include <iostream>
#include <boost/program_options.hpp>
#include "SpParameters.h"
#include "common.h"
#include "SvmPredictor.h"
#include <shogun/base/init.h>

namespace po = boost::program_options;

namespace
{
        void set_params(po::variables_map& vm, SpParameters& param); 
}

#ifndef UNIT_TEST
int main(int argc, const char * argv[])
{
        shogun::init_shogun_with_defaults();
        try {
                po::options_description desc("options"); 
                desc.add_options()
                        ("help", "show this help")
                        ("input-svm", po::value<std::string>(), "file path to svm serialization data")
                        ("data", po::value<std::string>(), "file path to data to classify")
                        ("output", po::value<std::string>(), "file path to output file")
                ;

                po::variables_map vm;
                po::store(po::parse_command_line(argc, argv, desc), vm);
                po::notify(vm);

                if ( vm.empty() || vm.size() != 3 || vm.count("help") ) {
                        std::cout << desc << std::endl;
                       return 1;
                }
                SpParameters params;
                set_params(vm, params);
                std::cout << params << std::endl;

                cv::Mat features;
                common::accumulate_cvmats(params.data_, features);

                SvmPredictor predictor;
                predictor.load(params.input_svm_);
                predictor.apply(features, params.output_);
                
        } catch (const std::exception& e) {
                std::cout << e.what() << std::endl;
        }
        shogun::exit_shogun();
        return 0;
}

namespace
{
        void set_params(po::variables_map& vm, SpParameters& params)
        {
                common::set_params(vm, "input-svm", params.input_svm_);
                common::set_params(vm, "data", params.data_);
                common::set_params(vm, "output", params.output_);
        }
        
        
}

#endif // UNIT_TEST
#ifdef UNIT_TEST
#define BOOST_AUTO_TEST_MAIN
#include <boost/test/unit_test.hpp>

BOOST_AUTO_TEST_CASE(TEST_main)
{       
        std::cout << "TEST_main:end\n";
}
#endif // UNIT_TEST